"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pipe = exports.getPath = exports.getFields = exports.addField = exports.removeId = exports.removeField = exports.getField = exports.notEquals = exports.equals = void 0;
function equals(value) {
    return (check) => check === value;
}
exports.equals = equals;
function notEquals(value) {
    return (check) => check !== value;
}
exports.notEquals = notEquals;
/**
 * Creates a function which will reduce an object type down to one of its named properties.
 *
 * @param field The name of a property for an object type.
 */
function getField(field) {
    return (obj) => obj[field];
}
exports.getField = getField;
/**
 * Creates a function which will reduce an object type down to one that excludes a named property.
 *
 * @param field The name of a property for an object type.
 */
function removeField(field) {
    return (obj) => Object.fromEntries(Object.keys(obj)
        .filter(notEquals(field))
        .map((field) => [field, obj[field]]));
}
exports.removeField = removeField;
exports.removeId = removeField('id');
/**
 * Creates a function which will map an object to a new one with an additional named property.
 *
 * @param field The name of a property for an object type.
 */
function addField(field) {
    return (value) => (obj) => ({ ...obj, [field]: value });
}
exports.addField = addField;
/**
 * Creates a function which will reduce an object type down to a subset of its named properties.
 *
 * @param fields A list of property names for an object type.
 */
function getFields(...fields) {
    return (obj) => Object.fromEntries(fields.map((field) => [field, obj[field]]));
}
exports.getFields = getFields;
/**
 * Works like getField except it can get properties for nested object fields.
 *
 * For example, if we have an object like: { A: { B: { C: true } } }
 * and we get these fields: A, B, C
 * then we will reduce the original object to A.B.C which is the value true.
 *
 * @param fields A list of property names
 */
function getPath(...fields) {
    return (obj) => fields.reduce((subObj, field) => (subObj && typeof subObj === 'object' ? subObj[field] : subObj), obj);
}
exports.getPath = getPath;
/**
 * Create a function that composes a list of functions
 *
 * @param fns A list of functions to be used in the pipe
 */
function pipe(...fns) {
    return (input) => fns.reduce((nextInput, fn) => fn(nextInput), input);
}
exports.pipe = pipe;
